import '../control/dfresponse.dart';
import 'dfbasefield.dart';

class AllowEmptyValidateException extends DfError {
   AllowEmptyValidateException() : super(code: 10001, message: "Value empty is not allowed");
}

class ListSizeValidateException extends DfError {
   ListSizeValidateException(DfValidatorListSize size, int currentSize) : super(code: 10002, message: "List size must be ${size.toString()}, but is $currentSize");
}

class DfValidatorListSize {
  int min;
  int max;

  DfValidatorListSize({this.min, this.max});

  String toString() {
    return "$min - $max";
  }
}

class DfValidator {

  DfBaseField field;
  bool isAllowEmpty = true;
  DfValidatorListSize listSize;

  DfValidator({this.isAllowEmpty = true, this.listSize});

  AllowEmptyValidateException emptyValidate() {
    if (!isAllowEmpty) {
      if (field.value == null) {
        return AllowEmptyValidateException();
      }
      else if (field.value is String && field.value == "") {
        return AllowEmptyValidateException();
      }
    }
    return null;
  }

  ListSizeValidateException listSizeValidate() {
    if (listSize != null && field.value is List) {
      if (field.value == null && listSize.min != null && listSize.min > 0) {
        return ListSizeValidateException(listSize,0);
      }
      else {
        List list = field.value;
        if ((listSize.min != null && list.length < listSize.min) || (listSize.max != null && list.length > listSize.max)) {
          return ListSizeValidateException(listSize,list.length);
        }
      }
    }
    return null;
  }

  List<DfError> validate() {
    List<DfError> errors = [];
    DfError emptyErr = emptyValidate();
    if (emptyErr != null) {
      errors.add(emptyErr);
    }
    DfError listSizeErr = listSizeValidate();
    if (listSizeErr != null) {
      errors.add(listSizeErr);
    }

    return errors.length > 0 ? errors : null;
  }

}